==> liquidwar6-0.0.1beta.tar.gz: liquidwar6-0.0.1beta/src/lib/gfx/mod-gl/gl-utils/gl-utils-texture.c <==
/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <string.h>

#include <GL/glu.h>

#include "config.h"
#include "../../gfx.h"
#include "gl-utils.h"
#include "gl-utils-internal.h"

/*
 * A bunch of code here has been ripped from SDL examples,
 * mostly testgl.c
 */

/* 
 * Quick utility function for texture creation
 */
int
mod_gl_utils_power_of_two_ge (int input)
{
  int value = 1;

  while (value < input)
    {
      value <<= 1;
    }
  return value;
}

int
mod_gl_utils_power_of_two_le (int input)
{
  int value = 1;

  while ((value << 1) <= input)
    {
      value <<= 1;
    }

  return value;
}

/*
 * Returns the scale to use in a given direction.
 */
float
mod_gl_utils_texture_scale (int size)
{
  return ((float) size) / ((float) mod_gl_utils_power_of_two_ge (size));
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
		      SDL_Surface * surface, int x, int y, int w, int h)
{
  GLuint texture = 0;

  SDL_Surface *image;
  SDL_Rect dst_area;
  SDL_Rect src_area;
  Uint32 saved_flags;
  Uint8 saved_alpha;

  if (w > gfx_context->caps.max_texture_size ||
      h > gfx_context->caps.max_texture_size)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _
		  ("texture is too big (%dx%d) for OpenGL driver (accepts %dx%d)"),
		  w, h, gfx_context->caps.max_texture_size,
		  gfx_context->caps.max_texture_size);
    }

  w = mod_gl_utils_power_of_two_ge (w);
  h = mod_gl_utils_power_of_two_ge (h);

  image = mod_gl_utils_create_surface (gfx_context, w, h);

  if (image == NULL)
    {
      return 0;
    }

  /* Save the alpha blending attributes */
  saved_flags = surface->flags & (SDL_SRCALPHA | SDL_RLEACCELOK);
  saved_alpha = surface->format->alpha;
  if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
    {
      SDL_SetAlpha (surface, 0, 0);
    }

  /* Copy the surface into the GL texture image */
  src_area.x = 0;
  src_area.y = 0;
  src_area.w = surface->w;
  src_area.h = surface->h;
  dst_area.x = x;
  dst_area.y = y;
  dst_area.w = surface->w - x;
  dst_area.h = surface->h - y;
  SDL_BlitSurface (surface, &src_area, image, &dst_area);

  /* Restore the alpha blending attributes */
  if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
    {
      SDL_SetAlpha (surface, saved_flags, saved_alpha);
    }

  /* Create an OpenGL texture for the image */
  glGenTextures (1, &texture);
  glBindTexture (GL_TEXTURE_2D, texture);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexImage2D (GL_TEXTURE_2D,
		0,
		GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

  if (glIsTexture (texture))
    {
      gfx_context->texture_counter.new_counter++;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to generate texture from surface"));
    }

  mod_gl_utils_delete_surface (gfx_context, image);	/* No longer needed */

  return texture;
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context, SDL_Surface * surface,
		    int w, int h)
{
  return surface2texture_xywh (gfx_context, surface, 0, 0, w, h);
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture (MOD_GL_UTILS_CONTEXT * gfx_context, SDL_Surface * surface)
{
  int w, h;

  /* Use the surface width and height expanded to powers of 2 */
  w = mod_gl_utils_power_of_two_ge (surface->w);
  h = mod_gl_utils_power_of_two_ge (surface->h);

  return surface2texture_wh (gfx_context, surface, w, h);
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
				   SDL_Surface * surface, int x, int y, int w,
				   int h)
{
  GLuint texture;

  texture = surface2texture_xywh (gfx_context, surface, x, y, w, h);

  return (int) texture;
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context,
				 SDL_Surface * surface, int w, int h)
{
  GLuint texture;

  texture = surface2texture_wh (gfx_context, surface, w, h);

  return (int) texture;
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			      SDL_Surface * surface)
{
  GLuint texture;

  texture = surface2texture (gfx_context, surface);

  return (int) texture;
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
map2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map, int x,
		  int y, int w, int h)
{
  GLuint texture = 0;
  SDL_Surface *image;

  if (w > gfx_context->caps.max_texture_size ||
      h > gfx_context->caps.max_texture_size)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _
		  ("map is too big (%dx%d) to be texturized by OpenGL driver"),
		  w, h);
    }

  w = mod_gl_utils_power_of_two_ge (w);
  h = mod_gl_utils_power_of_two_ge (h);

  image = mod_gl_utils_map2surface_xywh (gfx_context, map, x, y, w, h);

  if (image)
    {
      /* Create an OpenGL texture for the image */
      glGenTextures (1, &texture);
      glBindTexture (GL_TEXTURE_2D, texture);
      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
      glTexImage2D (GL_TEXTURE_2D,
		    0,
		    GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE,
		    image->pixels);

      mod_gl_utils_delete_surface (gfx_context, image);	/* No longer needed */
    }

  if (glIsTexture (texture))
    {
      gfx_context->texture_counter.new_counter++;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to generate texture from map"));
    }

  return texture;
}

static GLuint
map2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map, int w,
		int h)
{
  return map2texture_xywh (gfx_context, map, 0, 0, w, h);
}

static GLuint
map2texture (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map)
{
  int w, h;

  /* Use the map width and height expanded to powers of 2 */
  w = mod_gl_utils_power_of_two_ge (map->texture.shape.w);
  h = mod_gl_utils_power_of_two_ge (map->texture.shape.h);

  return map2texture_wh (gfx_context, map, w, h);
}

GLuint
mod_gl_utils_map2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
			       LW6MAP_MAP * map, int x, int y, int w, int h)
{
  GLuint texture;

  texture = map2texture_xywh (gfx_context, map, x, y, w, h);

  return (int) texture;
}

GLuint
mod_gl_utils_map2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context,
			     LW6MAP_MAP * map, int w, int h)
{
  GLuint texture;

  texture = map2texture_wh (gfx_context, map, w, h);

  return (int) texture;
}

GLuint
mod_gl_utils_map2texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			  LW6MAP_MAP * map)
{
  GLuint texture;

  texture = map2texture (gfx_context, map);

  return (int) texture;
}

/*
 * Deletes a texture.
 * Note that calling this directly from just after a glEnd
 * when you're drawing textured stuff might cause ugly unexpected
 * results. Not enough time to find out how OpenGL works internally,
 * my workarround is to create textures, use them, and when glFinish
 * has been called, then we delete all textures at once. In short:
 * use mod_gl_utils_schedule_delete_texture instead.
 */
void
mod_gl_utils_delete_texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			     GLuint texture)
{
  if (glIsTexture ((GLuint) texture))
    {
      gfx_context->texture_counter.delete_counter++;
      glDeleteTextures (1, (GLuint *) & texture);
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  "trying to delete texture, but it isn't a texture");
    }
}

/*
 * Schedules a texture to be deleted by mod_gl_utils_flush_delete_texture
 */
void
mod_gl_utils_schedule_delete_texture (MOD_GL_UTILS_CONTEXT * gfx_context,
				      GLuint texture)
{
  GLuint *texture_ptr = NULL;

  texture_ptr = LW6SYS_MALLOC (sizeof (GLuint));
  if (texture_ptr)
    {
      *texture_ptr = texture;
      lw6sys_list_push (&(gfx_context->texture_data.to_delete),
			(void *) texture_ptr);
    }
}

/*
 * Deletes all textures scheduled for delete.
 */
void
mod_gl_utils_delete_scheduled_textures (MOD_GL_UTILS_CONTEXT * gfx_context)
{
  GLuint *texture_ptr = NULL;

  while (!lw6sys_list_is_empty (gfx_context->texture_data.to_delete))
    {
      texture_ptr =
	(GLuint *) lw6sys_list_pop (&(gfx_context->texture_data.to_delete));
      if (texture_ptr != NULL)
	{
	  mod_gl_utils_delete_texture (gfx_context, *texture_ptr);
	  LW6SYS_FREE (texture_ptr);
	}
    }
}

void
mod_gl_utils_texture_update (MOD_GL_UTILS_CONTEXT *
			     utils_context,
			     GLuint texture, SDL_Surface * surface)
{
  /*
   * Note that texture and surface need be the same size, else KABOUM!
   * Usually, one should create the bitmap and the texture with
   * surface2texture then use this function afterwards on the same
   * objects. Safer. Typical use is displaying armies/fighters.
   */
  glBindTexture (GL_TEXTURE_2D, texture);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, surface->w, surface->h, GL_RGBA,
		   GL_UNSIGNED_BYTE, surface->pixels);
}

==> liquidwar6-0.0.2beta.tar.gz: liquidwar6-0.0.2beta/src/lib/gfx/mod-gl/gl-utils/gl-utils-texture.c <==
/*
Liquid War 6 is a unique multiplayer wargame.
Copyright (C)  2005, 2006, 2007  Christian Mauduit <ufoot@ufoot.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
Contact author        : ufoot@ufoot.org
*/

#include <stdio.h>
#include <string.h>

#include <GL/glu.h>

#include "config.h"
#include "../../gfx.h"
#include "gl-utils.h"
#include "gl-utils-internal.h"

/*
 * A bunch of code here has been ripped from SDL examples,
 * mostly testgl.c
 */

/* 
 * Quick utility function for texture creation
 */
int
mod_gl_utils_power_of_two_ge (int input)
{
  int value = 1;

  while (value < input)
    {
      value <<= 1;
    }
  return value;
}

int
mod_gl_utils_power_of_two_le (int input)
{
  int value = 1;

  while ((value << 1) <= input)
    {
      value <<= 1;
    }

  return value;
}

/*
 * Returns the scale to use in a given direction.
 */
float
mod_gl_utils_texture_scale (int size)
{
  return ((float) size) / ((float) mod_gl_utils_power_of_two_ge (size));
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
		      SDL_Surface * surface, int x, int y, int w, int h)
{
  GLuint texture = 0;

  SDL_Surface *image;
  SDL_Rect dst_area;
  SDL_Rect src_area;
  Uint32 saved_flags;
  Uint8 saved_alpha;

  if (w > gfx_context->caps.max_texture_size ||
      h > gfx_context->caps.max_texture_size)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _
		  ("texture is too big (%dx%d) for OpenGL driver (accepts %dx%d)"),
		  w, h, gfx_context->caps.max_texture_size,
		  gfx_context->caps.max_texture_size);
    }

  w = mod_gl_utils_power_of_two_ge (w);
  h = mod_gl_utils_power_of_two_ge (h);

  image = mod_gl_utils_create_surface (gfx_context, w, h);

  if (image == NULL)
    {
      return 0;
    }

  /* Save the alpha blending attributes */
  saved_flags = surface->flags & (SDL_SRCALPHA | SDL_RLEACCELOK);
  saved_alpha = surface->format->alpha;
  if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
    {
      SDL_SetAlpha (surface, 0, 0);
    }

  /* Copy the surface into the GL texture image */
  src_area.x = 0;
  src_area.y = 0;
  src_area.w = surface->w;
  src_area.h = surface->h;
  dst_area.x = x;
  dst_area.y = y;
  dst_area.w = surface->w - x;
  dst_area.h = surface->h - y;
  SDL_BlitSurface (surface, &src_area, image, &dst_area);

  /* Restore the alpha blending attributes */
  if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
    {
      SDL_SetAlpha (surface, saved_flags, saved_alpha);
    }

  /* Create an OpenGL texture for the image */
  glGenTextures (1, &texture);
  glBindTexture (GL_TEXTURE_2D, texture);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexImage2D (GL_TEXTURE_2D,
		0,
		GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

  if (glIsTexture (texture))
    {
      gfx_context->texture_counter.new_counter++;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to generate texture from surface"));
    }

  mod_gl_utils_delete_surface (gfx_context, image);	/* No longer needed */

  return texture;
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context, SDL_Surface * surface,
		    int w, int h)
{
  return surface2texture_xywh (gfx_context, surface, 0, 0, w, h);
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
surface2texture (MOD_GL_UTILS_CONTEXT * gfx_context, SDL_Surface * surface)
{
  int w, h;

  /* Use the surface width and height expanded to powers of 2 */
  w = mod_gl_utils_power_of_two_ge (surface->w);
  h = mod_gl_utils_power_of_two_ge (surface->h);

  return surface2texture_wh (gfx_context, surface, w, h);
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
				   SDL_Surface * surface, int x, int y, int w,
				   int h)
{
  GLuint texture;

  texture = surface2texture_xywh (gfx_context, surface, x, y, w, h);

  return (int) texture;
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context,
				 SDL_Surface * surface, int w, int h)
{
  GLuint texture;

  texture = surface2texture_wh (gfx_context, surface, w, h);

  return (int) texture;
}

/*
 * Converts a surface to a texture. Public wrapper function.
 */
GLuint
mod_gl_utils_surface2texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			      SDL_Surface * surface)
{
  GLuint texture;

  texture = surface2texture (gfx_context, surface);

  return (int) texture;
}

/*
 * Transforms an SDL bitmap into a proper GL texture.
 */
static GLuint
map2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map, int x,
		  int y, int w, int h)
{
  GLuint texture = 0;
  SDL_Surface *image;

  if (w > gfx_context->caps.max_texture_size ||
      h > gfx_context->caps.max_texture_size)
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _
		  ("map is too big (%dx%d) to be texturized by OpenGL driver"),
		  w, h);
    }

  w = mod_gl_utils_power_of_two_ge (w);
  h = mod_gl_utils_power_of_two_ge (h);

  image = mod_gl_utils_map2surface_xywh (gfx_context, map, x, y, w, h);

  if (image)
    {
      /* Create an OpenGL texture for the image */
      glGenTextures (1, &texture);
      glBindTexture (GL_TEXTURE_2D, texture);
      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
      glTexImage2D (GL_TEXTURE_2D,
		    0,
		    GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE,
		    image->pixels);

      mod_gl_utils_delete_surface (gfx_context, image);	/* No longer needed */
    }

  if (glIsTexture (texture))
    {
      gfx_context->texture_counter.new_counter++;
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  _("unable to generate texture from map"));
    }

  return texture;
}

static GLuint
map2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map, int w,
		int h)
{
  return map2texture_xywh (gfx_context, map, 0, 0, w, h);
}

static GLuint
map2texture (MOD_GL_UTILS_CONTEXT * gfx_context, LW6MAP_MAP * map)
{
  int w, h;

  /* Use the map width and height expanded to powers of 2 */
  w = mod_gl_utils_power_of_two_ge (map->texture.shape.w);
  h = mod_gl_utils_power_of_two_ge (map->texture.shape.h);

  return map2texture_wh (gfx_context, map, w, h);
}

GLuint
mod_gl_utils_map2texture_xywh (MOD_GL_UTILS_CONTEXT * gfx_context,
			       LW6MAP_MAP * map, int x, int y, int w, int h)
{
  GLuint texture;

  texture = map2texture_xywh (gfx_context, map, x, y, w, h);

  return (int) texture;
}

GLuint
mod_gl_utils_map2texture_wh (MOD_GL_UTILS_CONTEXT * gfx_context,
			     LW6MAP_MAP * map, int w, int h)
{
  GLuint texture;

  texture = map2texture_wh (gfx_context, map, w, h);

  return (int) texture;
}

GLuint
mod_gl_utils_map2texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			  LW6MAP_MAP * map)
{
  GLuint texture;

  texture = map2texture (gfx_context, map);

  return (int) texture;
}

/*
 * Deletes a texture.
 * Note that calling this directly from just after a glEnd
 * when you're drawing textured stuff might cause ugly unexpected
 * results. Not enough time to find out how OpenGL works internally,
 * my workarround is to create textures, use them, and when glFinish
 * has been called, then we delete all textures at once. In short:
 * use mod_gl_utils_schedule_delete_texture instead.
 */
void
mod_gl_utils_delete_texture (MOD_GL_UTILS_CONTEXT * gfx_context,
			     GLuint texture)
{
  if (glIsTexture ((GLuint) texture))
    {
      gfx_context->texture_counter.delete_counter++;
      glDeleteTextures (1, (GLuint *) & texture);
    }
  else
    {
      lw6sys_log (LW6SYS_LOG_WARNING, "gfx-mod-gl",
		  "trying to delete texture, but it isn't a texture");
    }
}

/*
 * Schedules a texture to be deleted by mod_gl_utils_flush_delete_texture
 */
void
mod_gl_utils_schedule_delete_texture (MOD_GL_UTILS_CONTEXT * gfx_context,
				      GLuint texture)
{
  GLuint *texture_ptr = NULL;

  texture_ptr = LW6SYS_MALLOC (sizeof (GLuint));
  if (texture_ptr)
    {
      *texture_ptr = texture;
      lw6sys_list_push (&(gfx_context->texture_data.to_delete),
			(void *) texture_ptr);
    }
}

/*
 * Deletes all textures scheduled for delete.
 */
void
mod_gl_utils_delete_scheduled_textures (MOD_GL_UTILS_CONTEXT * gfx_context)
{
  GLuint *texture_ptr = NULL;

  while (!lw6sys_list_is_empty (gfx_context->texture_data.to_delete))
    {
      texture_ptr =
	(GLuint *) lw6sys_list_pop (&(gfx_context->texture_data.to_delete));
      if (texture_ptr != NULL)
	{
	  mod_gl_utils_delete_texture (gfx_context, *texture_ptr);
	  LW6SYS_FREE (texture_ptr);
	}
    }
}

void
mod_gl_utils_texture_update (MOD_GL_UTILS_CONTEXT *
			     utils_context,
			     GLuint texture, SDL_Surface * surface)
{
  /*
   * Note that texture and surface need be the same size, else KABOUM!
   * Usually, one should create the bitmap and the texture with
   * surface2texture then use this function afterwards on the same
   * objects. Safer. Typical use is displaying armies/fighters.
   */
  glBindTexture (GL_TEXTURE_2D, texture);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, surface->w, surface->h, GL_RGBA,
		   GL_UNSIGNED_BYTE, surface->pixels);
}
